from django import forms
from django.core.validators import MaxValueValidator, MinValueValidator
from eveuniverse.models import EveSolarSystem, EveType

from buybackprogram.models import Owner, Program


class ProgramForm(forms.ModelForm):
    # specify the name of model to use
    class Meta:
        model = Program
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop("user", None)

        super(ProgramForm, self).__init__(*args, **kwargs)
        self.fields["owner"].queryset = Owner.objects.filter(user=self.user)


class ProgramItemForm(forms.Form):
    item_type = forms.ModelChoiceField(
        queryset=EveType.objects.none(),
        label="Item type",
        help_text="Add the name of item which you want to determine an tax on. Once you start typing, we offer suggestions",
        empty_label=None,
    )
    item_tax = forms.IntegerField(
        label="Tax amount",
        validators=[MaxValueValidator(100), MinValueValidator(-100)],
        help_text="Set an tax on the item. If program default tax is defined this tax will be added on top of the program tax. If program does not allow all items this tax is used to calculate the tax for the product.",
    )
    disallow_item = forms.BooleanField(
        label="Disallow item in this program",
        help_text="If you want to prevent any prices to be given for this item in this program you can check this box.",
        required=False,
    )

    def __init__(self, *args, **kwargs):
        value = kwargs.pop("value", None)

        super(ProgramItemForm, self).__init__(*args, **kwargs)

        if value is not None:
            self.fields["item_type"].queryset = EveType.objects.filter(
                pk=value,
                published=True,
            ).exclude(eve_group__eve_category__id=9)


class LocationForm(forms.Form):
    eve_solar_system = forms.ModelChoiceField(
        queryset=EveSolarSystem.objects.none(),
        label="Solar system",
        help_text="Select solar system name. Start typing and we will give you suggestions.",
        empty_label=None,
    )
    name = forms.CharField(
        label="Structure/station name",
        help_text="A name or identification tag of the structure where the items should be contracted at.",
    )

    def __init__(self, *args, **kwargs):
        value = kwargs.pop("value", None)

        super(LocationForm, self).__init__(*args, **kwargs)

        if value is not None:
            self.fields["eve_solar_system"].queryset = EveSolarSystem.objects.filter(
                pk=value,
            )


class CalculatorForm(forms.Form):

    items = forms.CharField(
        widget=forms.Textarea,
        label="Items",
        help_text="Copy and paste the item data from your inventory. Item types not in this buyback program will be ignored",
    )
    donation = forms.IntegerField(
        label="Donation %",
        initial=0,
        help_text="You can set a optional donation percentage on your contract",
        validators=[MaxValueValidator(100), MinValueValidator(0)],
    )



from typing import Any

from django.conf import settings
from django.contrib import messages
from django.contrib.messages.constants import DEBUG, ERROR, INFO, SUCCESS, WARNING
from django.utils.html import format_html

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None,
    choices: list = None,
) -> Any:
    """cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overriden)

    `None` allowed as value

    Returns cleaned value for setting
    """
    if default_value is None and not required_type:
        raise ValueError("You must specify a required_type for None defaults")

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        dirty_value = getattr(settings, name)
        if dirty_value is None or (
            isinstance(dirty_value, required_type)
            and (min_value is None or dirty_value >= min_value)
            and (max_value is None or dirty_value <= max_value)
            and (choices is None or dirty_value in choices)
        ):
            cleaned_value = dirty_value
        else:
            logger.warn(
                "You setting for {} it not valid. Please correct it. "
                "Using default for now: {}".format(name, default_value)
            )
            cleaned_value = default_value
    return cleaned_value


class messages_plus:
    """Pendant to Django messages adding level icons and HTML support

    Careful: Use with safe strings only
    """

    @classmethod
    def _add_messages_icon(cls, level: int, message: str) -> str:

        return format_html(
            "{}",
            message,
        )

    @classmethod
    def debug(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.debug(
            request, cls._add_messages_icon(DEBUG, message), extra_tags, fail_silently
        )

    @classmethod
    def info(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.info(
            request, cls._add_messages_icon(INFO, message), extra_tags, fail_silently
        )

    @classmethod
    def success(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.success(
            request, cls._add_messages_icon(SUCCESS, message), extra_tags, fail_silently
        )

    @classmethod
    def warning(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.warning(
            request, cls._add_messages_icon(WARNING, message), extra_tags, fail_silently
        )

    @classmethod
    def error(
        cls,
        request: object,
        message: str,
        extra_tags: str = "",
        fail_silently: bool = False,
    ):
        messages.error(
            request, cls._add_messages_icon(ERROR, message), extra_tags, fail_silently
        )



from django import template

from buybackprogram.models import ProgramItem

register = template.Library()


def setting_icons(option, program):
    restriction_groups = [v.name for v in program.restricted_to_group.all()]
    restriction_states = [v.name for v in program.restricted_to_state.all()]

    icons = {
        "all_items": {
            "icon": "fa-globe-europe",
            "color": False,
            "message": "This program accepts all types of items.",
        },
        "fuel_cost": {
            "icon": "fa-truck",
            "color": False,
            "message": "Items sold via this program have an added freight cost of %s ISK per m³"
            % program.hauling_fuel_cost,
        },
        "price_dencity_modifier": {
            "icon": "fa-compress-arrows-alt",
            "color": False,
            "message": "Items with price dencity bellow {} isk/m³ will have an additional {} % tax on them.".format(
                program.price_dencity_treshold, program.price_dencity_tax
            ),
        },
        "compressed": {
            "icon": "fa-file-archive",
            "color": False,
            "message": "Compressed price is taken into account when calculating values for ores.",
        },
        "refined": {
            "icon": "fa-industry",
            "color": False,
            "message": "Refined price is taken into account when calculating values for ores.",
        },
        "raw_ore": {
            "icon": "fa-icicles",
            "color": False,
            "message": "Raw price is taken into account when calculating values for ores.",
        },
        "unpacked": {
            "icon": "fa-box-open",
            "color": False,
            "message": "Unpacked items are accepted in this program.",
        },
        "special_items": {
            "icon": "fa-search-dollar",
            "color": False,
            "message": "This program has individual items with adjusted taxes or items that are not allowed in the program.",
        },
        "restricted_to_group": {
            "icon": "fa-user",
            "color": False,
            "message": "Only users in the following groups can access this program: %s"
            % ", ".join(restriction_groups),
        },
        "restricted_to_state": {
            "icon": "fa-users",
            "color": False,
            "message": "Only users in the following states can access this program: %s"
            % ", ".join(restriction_states),
        },
    }

    return icons[option]


@register.filter
def program_setting(program):

    settings = []

    program_items = ProgramItem.objects.filter(program=program)

    if program.allow_all_items:
        setting = setting_icons("all_items", program)

        settings.append(setting)

    if program.hauling_fuel_cost:
        setting = setting_icons("fuel_cost", program)
        settings.append(setting)

    if program.price_dencity_modifier:
        setting = setting_icons("price_dencity_modifier", program)

        settings.append(setting)

    if program.use_compressed_value:
        setting = setting_icons("compressed", program)

        settings.append(setting)

    if program.use_refined_value:
        setting = setting_icons("refined", program)

        settings.append(setting)

    if program.use_raw_ore_value:
        setting = setting_icons("raw_ore", program)

        settings.append(setting)

    if program.allow_unpacked_items:
        setting = setting_icons("unpacked", program)

        settings.append(setting)

    if program_items:
        setting = setting_icons("special_items", program)

        settings.append(setting)

    if program.restricted_to_group.all():
        setting = setting_icons("restricted_to_group", program)

        settings.append(setting)

    if program.restricted_to_state.all():
        setting = setting_icons("restricted_to_state", program)

        settings.append(setting)

    return settings



from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import F, Q
from django.http import JsonResponse
from django.shortcuts import render
from eveuniverse.models import EveSolarSystem, EveType

from allianceauth.services.hooks import get_extension_logger

from buybackprogram.models import Program

logger = get_extension_logger(__name__)


@login_required
@permission_required("buybackprogram.basic_access")
def index(request):

    user_groups = request.user.groups.all()

    logger.debug("User %s has groups: %s" % (request.user, user_groups))

    user_state = [request.user.profile.state]

    logger.debug("User %s state is : %s" % (request.user, user_state))

    program = (
        Program.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True)
            | Q(owner__user=request.user)
        )
        .filter(
            Q(restricted_to_state=request.user.profile.state)
            | Q(restricted_to_state__isnull=True)
            | Q(owner__user=request.user)
        )
        .distinct()
    )

    context = {"programs": program}

    return render(request, "buybackprogram/index.html", context)


@login_required
@permission_required("buybackprogram.manage_programs")
def item_autocomplete(request):
    items = EveType.objects.filter(published=True).exclude(
        eve_group__eve_category__id=9
    )

    q = request.GET.get("q", None)

    if q is not None:
        items = items.filter(name__icontains=q)

    items = items.annotate(
        value=F("id"),
        text=F("name"),
    ).values("value", "text")

    return JsonResponse(list(items), safe=False)


@login_required
@permission_required("buybackprogram.manage_programs")
def solarsystem_autocomplete(request):
    items = EveSolarSystem.objects.all()

    q = request.GET.get("q", None)

    if q is not None:
        items = items.filter(name__icontains=q)

    items = items.annotate(
        value=F("id"),
        text=F("name"),
    ).values("value", "text")

    return JsonResponse(list(items), safe=False)
